@@ -1,6 +1,7 @@
 package reader
 
 import (
+	"math"
 	"os"
 	"os/exec"
 	"path"
@@ -27,42 +28,42 @@ func init() {
 }
 
 func testGetLineCount(t *testing.T, reader *ReaderImpl) {
-	if strings.Contains(*reader.Name, "compressed") {
+	if strings.Contains(*reader.DisplayName, "compressed") {
 		// We are no good at counting lines of compressed files, never mind
 		return
 	}
 
-	cmd := exec.Command("wc", "-l", *reader.Name)
+	cmd := exec.Command("wc", "-l", *reader.FileName)
 	output, err := cmd.CombinedOutput()
 	if err != nil {
-		t.Error("Error calling wc -l to count lines of", *reader.Name, err)
+		t.Error("Error calling wc -l to count lines of", *reader.FileName, err)
 	}
 
 	wcNumberString := strings.Split(strings.TrimSpace(string(output)), " ")[0]
 	wcLineCount, err := strconv.Atoi(wcNumberString)
 	if err != nil {
-		t.Error("Error counting lines of", *reader.Name, err)
+		t.Error("Error counting lines of", *reader.FileName, err)
 	}
 
 	// wc -l under-counts by 1 if the file doesn't end in a newline
-	rawBytes, err := os.ReadFile(*reader.Name)
+	rawBytes, err := os.ReadFile(*reader.FileName)
 	if err == nil && len(rawBytes) > 0 && rawBytes[len(rawBytes)-1] != '\n' {
 		wcLineCount++
 	}
 
 	if reader.GetLineCount() != wcLineCount {
 		t.Errorf("Got %d lines from the reader but %d lines from wc -l: <%s>",
-			reader.GetLineCount(), wcLineCount, *reader.Name)
+			reader.GetLineCount(), wcLineCount, *reader.FileName)
 	}
 
-	countLinesCount, err := countLines(*reader.Name)
+	countLinesCount, err := countLines(*reader.FileName)
 	assert.NilError(t, err)
 	if countLinesCount != uint64(wcLineCount) {
 		t.Errorf("Got %d lines from wc -l, but %d lines from our countLines() function", wcLineCount, countLinesCount)
 	}
 }
 
-func firstLine(inputLines *InputLines) linemetadata.Index {
+func firstLine(inputLines InputLines) linemetadata.Index {
 	return inputLines.Lines[0].Index
 }
 
@@ -316,7 +317,8 @@ func TestReadTextDone(t *testing.T) {
 
 // JSON should be auto detected and formatted
 func TestFormatJson(t *testing.T) {
-	jsonStream := strings.NewReader(`{"key": "value"}`)
+	// Note the space after "key" to verify formatting actually happens
+	jsonStream := strings.NewReader(`{"key" :"value"}`)
 	testMe, err := NewFromStream(
 		"JSON test",
 		jsonStream,
@@ -337,7 +339,8 @@ func TestFormatJson(t *testing.T) {
 }
 
 func TestFormatJsonArray(t *testing.T) {
-	jsonStream := strings.NewReader(`[{"key": "value"}]`)
+	// Note the space after "key" to verify formatting actually happens
+	jsonStream := strings.NewReader(`[{"key" :"value"}]`)
 	testMe, err := NewFromStream(
 		"JSON test",
 		jsonStream,
@@ -564,6 +567,43 @@ func TestReadUpdatingFile_HalfUtf8(t *testing.T) {
 	assert.Equal(t, int(testMe.bytesCount), len([]byte("här")))
 }
 
+func TestClipRangeToLength(t *testing.T) {
+	// Within bounds
+	i0, i1 := clipRangeToLength(linemetadata.Index{}, 1, 20)
+	assert.Equal(t, i0, 0)
+	assert.Equal(t, i1, 0)
+
+	// Touching the end, still within bounds
+	i0, i1 = clipRangeToLength(linemetadata.Index{}, 1, 0)
+	assert.Equal(t, i0, 0)
+	assert.Equal(t, i1, 0)
+
+	// Overflow, push down to indices 6, 7, 8
+	i0, i1 = clipRangeToLength(linemetadata.IndexFromOneBased(100), 3, 8)
+	assert.Equal(t, i0, 6)
+	assert.Equal(t, i1, 8)
+
+	// Overflow, push down and clip to indices 0, 1
+	i0, i1 = clipRangeToLength(linemetadata.IndexFromOneBased(100), 3, 1)
+	assert.Equal(t, i0, 0)
+	assert.Equal(t, i1, 1)
+
+	// Maxed out start
+	i0, i1 = clipRangeToLength(linemetadata.IndexMax(), 1, 0)
+	assert.Equal(t, i0, 0)
+	assert.Equal(t, i1, 0)
+
+	// Maxed out count
+	i0, i1 = clipRangeToLength(linemetadata.Index{}, math.MaxInt, 0)
+	assert.Equal(t, i0, 0)
+	assert.Equal(t, i1, 0)
+
+	// Maxed out start and count
+	i0, i1 = clipRangeToLength(linemetadata.IndexMax(), math.MaxInt, 3)
+	assert.Equal(t, i0, 0)
+	assert.Equal(t, i1, 3)
+}
+
 // How long does it take to read a file?
 //
 // This can be slow due to highlighting.
@@ -610,6 +650,8 @@ func BenchmarkReadLargeFile(b *testing.B) {
 	// Make sure we don't pause during the benchmark
 	targetLineCount := largeSizeBytes * 2
 
+	b.SetBytes(int64(totalBytesWritten))
+
 	b.ResetTimer()
 	for n := 0; n < b.N; n++ {
 		readMe, err := NewFromFilename(
