@@ -88,6 +88,29 @@ func (sp scrollPosition) NextLine(scrollDistance int) scrollPosition {
 	}
 }
 
+func (p *Pager) ScrollPositionsEqual(a, b scrollPosition) bool {
+	a.internalDontTouch.canonicalize(p)
+	b.internalDontTouch.canonicalize(p)
+	if a.internalDontTouch.deltaScreenLines != b.internalDontTouch.deltaScreenLines {
+		return false
+	}
+
+	if (a.internalDontTouch.lineIndex == nil) != (b.internalDontTouch.lineIndex == nil) {
+		// One line index is nil, the other is not
+		return false
+	}
+
+	if a.internalDontTouch.lineIndex != nil && b.internalDontTouch.lineIndex != nil {
+		// Both line indexes are non-nil...
+		if *a.internalDontTouch.lineIndex != *b.internalDontTouch.lineIndex {
+			// ... but they point to different lines.
+			return false
+		}
+	}
+
+	return true
+}
+
 // Create a new position, scrolled to the given line number
 //
 //revive:disable-next-line:unexported-return
@@ -109,7 +132,7 @@ func (si *scrollPositionInternal) handleNegativeDeltaScreenLines(pager *Pager) {
 		previousLine := pager.Reader().GetLine(previousLineIndex)
 		previousSubLinesCount := 0
 		if previousLine != nil {
-			previousSubLines := pager.renderLine(previousLine, si.getMaxNumberPrefixLength(pager))
+			previousSubLines := pager.renderLine(*previousLine, si.getMaxNumberPrefixLength(pager), true)
 			previousSubLinesCount = len(previousSubLines)
 		}
 
@@ -143,14 +166,14 @@ func (si *scrollPositionInternal) handlePositiveDeltaScreenLines(pager *Pager) {
 			if line == nil {
 				panic(fmt.Errorf("Last line is nil"))
 			}
-			subLines := pager.renderLine(line, maxPrefixLength)
+			subLines := pager.renderLine(*line, maxPrefixLength, true)
 
 			// ... and go to the bottom of that.
 			si.deltaScreenLines = len(subLines) - 1
 			return
 		}
 
-		subLines := pager.renderLine(line, maxPrefixLength)
+		subLines := pager.renderLine(*line, maxPrefixLength, true)
 		if si.deltaScreenLines < len(subLines) {
 			// Sublines are within bounds!
 			return
@@ -189,7 +212,7 @@ func (si *scrollPositionInternal) emptyBottomLinesCount(pager *Pager) int {
 			break
 		}
 
-		subLines := pager.renderLine(line, lastLineNumberWidth)
+		subLines := pager.renderLine(*line, lastLineNumberWidth, true)
 		unclaimedViewportLines -= len(subLines)
 		if unclaimedViewportLines <= 0 {
 			return 0
@@ -300,7 +323,7 @@ func (p *Pager) scrollToEnd() {
 
 	// Scroll down enough. We know for sure the last line won't wrap into more
 	// lines than the number of characters it contains.
-	p.scrollPosition.internalDontTouch.deltaScreenLines = len(lastInputLine.Line.Plain(&lastInputLine.Index))
+	p.scrollPosition.internalDontTouch.deltaScreenLines = len(lastInputLine.Line.Plain(lastInputIndex))
 
 	if p.TargetLine == nil {
 		// Start following the end of the file
@@ -332,7 +355,7 @@ func (p *Pager) isScrolledToEnd() bool {
 	// Last line is on screen, now we need to figure out whether we can see all
 	// of it
 	lastInputLine := p.Reader().GetLine(lastInputLineIndex)
-	lastInputLineRendered := p.renderLine(lastInputLine, p.getLineNumberPrefixLength(lastInputLine.Number))
+	lastInputLineRendered := p.renderLine(*lastInputLine, p.getLineNumberPrefixLength(lastInputLine.Number), true)
 	lastRenderedSubLine := lastInputLineRendered[len(lastInputLineRendered)-1]
 
 	// If the last visible subline is the same as the last possible subline then
@@ -371,8 +394,18 @@ func (si *scrollPositionInternal) getMaxNumberPrefixLength(pager *Pager) int {
 	if si.lineIndex != nil {
 		index = *si.lineIndex
 	}
+
+	// Not going back when computing the max number prefix length prevents
+	// https://github.com/walles/moor/issues/338.
+	//
+	// FIXME: Is there something better we should be doing instead?
+	positiveDeltaScreenLines := si.deltaScreenLines
+	if positiveDeltaScreenLines < 0 {
+		positiveDeltaScreenLines = 0
+	}
+
 	maxVisibleIndex := index.NonWrappingAdd(
-		si.deltaScreenLines +
+		positiveDeltaScreenLines +
 			pager.visibleHeight() - 1)
 	if maxVisibleIndex.IsAfter(maxPossibleIndex) {
 		maxVisibleIndex = maxPossibleIndex
